package sun.security.provider.certpath.ldap;

import java.security.PublicKey;
import java.util.Date;
import java.math.BigInteger;
import java.io.IOException;
import sun.security.x509.X500Name;
import java.util.Locale;
import javax.naming.NamingEnumeration;
import javax.naming.directory.Attribute;
import javax.naming.NameNotFoundException;
import com.sun.jndi.ldap.LdapReferralException;
import javax.naming.ldap.LdapContext;
import java.util.HashMap;
import javax.naming.InvalidNameException;
import javax.naming.CompositeName;
import java.util.List;
import java.util.Map;
import sun.security.action.GetPropertyAction;
import javax.naming.directory.BasicAttributes;
import java.net.URI;
import java.util.Set;
import javax.security.auth.x500.X500Principal;
import java.security.cert.CRLSelector;
import java.security.cert.CRL;
import java.security.cert.CRLException;
import java.security.cert.X509CRL;
import java.security.cert.X509CRLSelector;
import java.util.HashSet;
import java.security.cert.CertSelector;
import java.util.Iterator;
import sun.security.provider.certpath.X509CertificatePair;
import java.security.cert.Certificate;
import java.util.Collections;
import sun.misc.HexDumpEncoder;
import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.security.cert.CertStoreException;
import java.security.cert.X509Certificate;
import java.util.Collection;
import java.security.cert.X509CertSelector;
import javax.naming.NamingException;
import javax.naming.directory.InitialDirContext;
import java.security.AccessController;
import sun.security.action.GetBooleanAction;
import java.util.Hashtable;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.InvalidAlgorithmParameterException;
import java.security.cert.CertStoreParameters;
import java.security.cert.CertStore;
import java.security.cert.LDAPCertStoreParameters;
import sun.security.util.Cache;
import javax.naming.directory.DirContext;
import java.security.cert.CertificateFactory;
import javax.naming.directory.Attributes;
import sun.security.util.Debug;
import java.security.cert.CertStoreSpi;

public final class LDAPCertStore extends CertStoreSpi
{
    private static final Debug debug;
    private static final boolean DEBUG = false;
    private static final String USER_CERT = "userCertificate;binary";
    private static final String CA_CERT = "cACertificate;binary";
    private static final String CROSS_CERT = "crossCertificatePair;binary";
    private static final String CRL = "certificateRevocationList;binary";
    private static final String ARL = "authorityRevocationList;binary";
    private static final String DELTA_CRL = "deltaRevocationList;binary";
    private static final String[] STRING0;
    private static final byte[][] BB0;
    private static final Attributes EMPTY_ATTRIBUTES;
    private static final int DEFAULT_CACHE_SIZE = 750;
    private static final int DEFAULT_CACHE_LIFETIME = 30;
    private static final int LIFETIME;
    private static final String PROP_LIFETIME = "sun.security.certpath.ldap.cache.lifetime";
    private static final String PROP_DISABLE_APP_RESOURCE_FILES = "sun.security.certpath.ldap.disable.app.resource.files";
    private CertificateFactory cf;
    private DirContext ctx;
    private boolean prefetchCRLs;
    private final Cache<String, byte[][]> valueCache;
    private int cacheHits;
    private int cacheMisses;
    private int requests;
    private static final Cache<LDAPCertStoreParameters, CertStore> certStoreCache;
    
    public LDAPCertStore(CertStoreParameters params) throws InvalidAlgorithmParameterException {
        super(params);
        this.prefetchCRLs = false;
        this.cacheHits = 0;
        this.cacheMisses = 0;
        this.requests = 0;
        if (!(params instanceof LDAPCertStoreParameters)) {
            throw new InvalidAlgorithmParameterException("parameters must be LDAPCertStoreParameters");
        }
        LDAPCertStoreParameters ldapCertStoreParameters = (LDAPCertStoreParameters)params;
        this.createInitialDirContext(ldapCertStoreParameters.getServerName(), ldapCertStoreParameters.getPort());
        try {
            this.cf = CertificateFactory.getInstance("X.509");
        }
        catch (CertificateException ex) {
            throw new InvalidAlgorithmParameterException("unable to create CertificateFactory for X.509");
        }
        if (LDAPCertStore.LIFETIME == 0) {
            this.valueCache = Cache.newNullCache();
        }
        else if (LDAPCertStore.LIFETIME < 0) {
            this.valueCache = Cache.newSoftMemoryCache(750);
        }
        else {
            this.valueCache = Cache.newSoftMemoryCache(750, LDAPCertStore.LIFETIME);
        }
        return;
    }
    
    static synchronized CertStore getInstance(LDAPCertStoreParameters params) throws NoSuchAlgorithmException, InvalidAlgorithmParameterException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkConnect(params.getServerName(), params.getPort());
        }
        CertStore instance = (CertStore)LDAPCertStore.certStoreCache.get(params);
        if (instance == null) {
            instance = CertStore.getInstance("LDAP", params);
            LDAPCertStore.certStoreCache.put(params, instance);
        }
        else if (LDAPCertStore.debug != null) {
            LDAPCertStore.debug.println("LDAPCertStore.getInstance: cache hit");
        }
        return instance;
    }
    
    private void createInitialDirContext(String str, int i) throws InvalidAlgorithmParameterException {
        String string = "ldap://" + str + ":" + i;
        Hashtable<String, String> environment = new Hashtable();
        environment.put("java.naming.factory.initial", "com.sun.jndi.ldap.LdapCtxFactory");
        environment.put("java.naming.provider.url", string);
        if ((boolean)AccessController.doPrivileged(new GetBooleanAction("sun.security.certpath.ldap.disable.app.resource.files"))) {
            if (LDAPCertStore.debug != null) {
                LDAPCertStore.debug.println("LDAPCertStore disabling app resource files");
            }
            environment.put("com.sun.naming.disable.app.resource.files", "true");
        }
        try {
            this.ctx = new InitialDirContext(environment);
            if (this.ctx.getEnvironment().get("java.naming.referral") == null) {
                this.ctx.addToEnvironment("java.naming.referral", "throw");
            }
        }
        catch (NamingException cause) {
            if (LDAPCertStore.debug != null) {
                LDAPCertStore.debug.println("LDAPCertStore.engineInit about to throw InvalidAlgorithmParameterException");
                cause.printStackTrace();
            }
            InvalidAlgorithmParameterException ex = new InvalidAlgorithmParameterException("unable to create InitialDirContext using supplied parameters");
            ex.initCause(cause);
            throw (InvalidAlgorithmParameterException)ex;
        }
    }
    
    private Collection<X509Certificate> getCertificates(LDAPCertStore.LDAPRequest ldapRequest, String s, X509CertSelector x509CertSelector) throws CertStoreException {
        byte[][] values;
        try {
            values = ldapRequest.getValues(s);
        }
        catch (NamingException cause) {
            throw new CertStoreException(cause);
        }
        int length = values.length;
        if (length == 0) {
            return Collections.emptySet();
        }
        ArrayList<X509Certificate> list = new ArrayList(length);
        for (int i = 0; i < length; ++i) {
            ByteArrayInputStream inStream = new ByteArrayInputStream(values[i]);
            try {
                Certificate generateCertificate = this.cf.generateCertificate(inStream);
                if (x509CertSelector.match(generateCertificate)) {
                    list.add((X509Certificate)generateCertificate);
                }
            }
            catch (CertificateException ex) {
                if (LDAPCertStore.debug != null) {
                    LDAPCertStore.debug.println("LDAPCertStore.getCertificates() encountered exception while parsing cert, skipping the bad data: ");
                    LDAPCertStore.debug.println("[ " + new HexDumpEncoder().encodeBuffer(values[i]) + " ]");
                }
            }
        }
        return list;
    }
    
    private Collection<X509CertificatePair> getCertPairs(LDAPCertStore.LDAPRequest ldapRequest, String s) throws CertStoreException {
        byte[][] values;
        try {
            values = ldapRequest.getValues(s);
        }
        catch (NamingException cause) {
            throw new CertStoreException(cause);
        }
        int length = values.length;
        if (length == 0) {
            return Collections.emptySet();
        }
        ArrayList<X509CertificatePair> list = new ArrayList(length);
        for (int i = 0; i < length; ++i) {
            try {
                list.add(X509CertificatePair.generateCertificatePair(values[i]));
            }
            catch (CertificateException ex) {
                if (LDAPCertStore.debug != null) {
                    LDAPCertStore.debug.println("LDAPCertStore.getCertPairs() encountered exception while parsing cert, skipping the bad data: ");
                    LDAPCertStore.debug.println("[ " + new HexDumpEncoder().encodeBuffer(values[i]) + " ]");
                }
            }
        }
        return list;
    }
    
    private Collection<X509Certificate> getMatchingCrossCerts(LDAPCertStore.LDAPRequest ldapRequest, X509CertSelector x509CertSelector, X509CertSelector x509CertSelector2) throws CertStoreException {
        Collection<X509CertificatePair> certPairs = this.getCertPairs(ldapRequest, "crossCertificatePair;binary");
        ArrayList<X509Certificate> list = new ArrayList();
        for (X509CertificatePair x509CertificatePair : certPairs) {
            if (x509CertSelector != null) {
                X509Certificate forward = x509CertificatePair.getForward();
                if (forward != null && x509CertSelector.match(forward)) {
                    list.add(forward);
                }
            }
            if (x509CertSelector2 != null) {
                X509Certificate reverse = x509CertificatePair.getReverse();
                if (reverse != null && x509CertSelector2.match(reverse)) {
                    list.add(reverse);
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        return list;
    }
    
    @Override
    public synchronized Collection<X509Certificate> engineGetCertificates(CertSelector obj) throws CertStoreException {
        if (LDAPCertStore.debug != null) {
            LDAPCertStore.debug.println("LDAPCertStore.engineGetCertificates() selector: " + String.valueOf(obj));
        }
        if (obj == null) {
            obj = new X509CertSelector();
        }
        if (!(obj instanceof X509CertSelector)) {
            throw new CertStoreException("LDAPCertStore needs an X509CertSelector to find certs");
        }
        X509CertSelector x509CertSelector = (X509CertSelector)obj;
        int basicConstraints = x509CertSelector.getBasicConstraints();
        String subjectAsString = x509CertSelector.getSubjectAsString();
        String issuerAsString = x509CertSelector.getIssuerAsString();
        HashSet o = new HashSet();
        if (LDAPCertStore.debug != null) {
            LDAPCertStore.debug.println("LDAPCertStore.engineGetCertificates() basicConstraints: " + basicConstraints);
        }
        if (subjectAsString != null) {
            if (LDAPCertStore.debug != null) {
                LDAPCertStore.debug.println("LDAPCertStore.engineGetCertificates() subject is not null");
            }
            LDAPCertStore.LDAPRequest o2 = new LDAPCertStore.LDAPRequest(subjectAsString);
            if (basicConstraints > -2) {
                o2.addRequestedAttribute("crossCertificatePair;binary");
                o2.addRequestedAttribute("cACertificate;binary");
                o2.addRequestedAttribute("authorityRevocationList;binary");
                if (this.prefetchCRLs) {
                    o2.addRequestedAttribute("certificateRevocationList;binary");
                }
            }
            if (basicConstraints < 0) {
                o2.addRequestedAttribute("userCertificate;binary");
            }
            if (basicConstraints > -2) {
                o.addAll(this.getMatchingCrossCerts(o2, x509CertSelector, null));
                if (LDAPCertStore.debug != null) {
                    LDAPCertStore.debug.println("LDAPCertStore.engineGetCertificates() after getMatchingCrossCerts(subject,xsel,null),certs.size(): " + o.size());
                }
                o.addAll(this.getCertificates(o2, "cACertificate;binary", x509CertSelector));
                if (LDAPCertStore.debug != null) {
                    LDAPCertStore.debug.println("LDAPCertStore.engineGetCertificates() after getCertificates(subject,CA_CERT,xsel),certs.size(): " + o.size());
                }
            }
            if (basicConstraints < 0) {
                o.addAll(this.getCertificates(o2, "userCertificate;binary", x509CertSelector));
                if (LDAPCertStore.debug != null) {
                    LDAPCertStore.debug.println("LDAPCertStore.engineGetCertificates() after getCertificates(subject,USER_CERT, xsel),certs.size(): " + o.size());
                }
            }
        }
        else {
            if (LDAPCertStore.debug != null) {
                LDAPCertStore.debug.println("LDAPCertStore.engineGetCertificates() subject is null");
            }
            if (basicConstraints == -2) {
                throw new CertStoreException("need subject to find EE certs");
            }
            if (issuerAsString == null) {
                throw new CertStoreException("need subject or issuer to find certs");
            }
        }
        if (LDAPCertStore.debug != null) {
            LDAPCertStore.debug.println("LDAPCertStore.engineGetCertificates() about to getMatchingCrossCerts...");
        }
        if (issuerAsString != null && basicConstraints > -2) {
            LDAPCertStore.LDAPRequest o3 = new LDAPCertStore.LDAPRequest(issuerAsString);
            o3.addRequestedAttribute("crossCertificatePair;binary");
            o3.addRequestedAttribute("cACertificate;binary");
            o3.addRequestedAttribute("authorityRevocationList;binary");
            if (this.prefetchCRLs) {
                o3.addRequestedAttribute("certificateRevocationList;binary");
            }
            o.addAll(this.getMatchingCrossCerts(o3, null, x509CertSelector));
            if (LDAPCertStore.debug != null) {
                LDAPCertStore.debug.println("LDAPCertStore.engineGetCertificates() after getMatchingCrossCerts(issuer,null,xsel),certs.size(): " + o.size());
            }
            o.addAll(this.getCertificates(o3, "cACertificate;binary", x509CertSelector));
            if (LDAPCertStore.debug != null) {
                LDAPCertStore.debug.println("LDAPCertStore.engineGetCertificates() after getCertificates(issuer,CA_CERT,xsel),certs.size(): " + o.size());
            }
        }
        if (LDAPCertStore.debug != null) {
            LDAPCertStore.debug.println("LDAPCertStore.engineGetCertificates() returning certs");
            return o;
        }
        return o;
    }
    
    private Collection<X509CRL> getCRLs(LDAPCertStore.LDAPRequest ldapRequest, String s, X509CRLSelector x509CRLSelector) throws CertStoreException {
        byte[][] values;
        try {
            values = ldapRequest.getValues(s);
        }
        catch (NamingException cause) {
            throw new CertStoreException(cause);
        }
        int length = values.length;
        if (length == 0) {
            return Collections.emptySet();
        }
        ArrayList<X509CRL> list = new ArrayList(length);
        for (int i = 0; i < length; ++i) {
            try {
                CRL generateCRL = this.cf.generateCRL(new ByteArrayInputStream(values[i]));
                if (x509CRLSelector.match(generateCRL)) {
                    list.add((X509CRL)generateCRL);
                }
            }
            catch (CRLException ex) {
                if (LDAPCertStore.debug != null) {
                    LDAPCertStore.debug.println("LDAPCertStore.getCRLs() encountered exception while parsing CRL, skipping the bad data: ");
                    LDAPCertStore.debug.println("[ " + new HexDumpEncoder().encodeBuffer(values[i]) + " ]");
                }
            }
        }
        return list;
    }
    
    @Override
    public synchronized Collection<X509CRL> engineGetCRLs(CRLSelector obj) throws CertStoreException {
        if (LDAPCertStore.debug != null) {
            LDAPCertStore.debug.println("LDAPCertStore.engineGetCRLs() selector: " + obj);
        }
        if (obj == null) {
            obj = new X509CRLSelector();
        }
        if (!(obj instanceof X509CRLSelector)) {
            throw new CertStoreException("need X509CRLSelector to find CRLs");
        }
        X509CRLSelector x509CRLSelector = (X509CRLSelector)obj;
        HashSet o = new HashSet();
        X509Certificate certificateChecking = x509CRLSelector.getCertificateChecking();
        Object issuerNames;
        if (certificateChecking != null) {
            issuerNames = new HashSet();
            issuerNames.add(certificateChecking.getIssuerX500Principal().getName("RFC2253"));
        }
        else {
            issuerNames = x509CRLSelector.getIssuerNames();
            if (issuerNames == null) {
                throw new CertStoreException("need issuerNames or certChecking to find CRLs");
            }
        }
        for (Object next : issuerNames) {
            String name;
            if ((next instanceof byte[])) {
                try {
                    name = new X500Principal((byte[])(byte[])next).getName("RFC2253");
                }
                catch (IllegalArgumentException ex) {
                    continue;
                }
            }
            else {
                name = (String)next;
            }
            Set<X509CRL> c = Collections.emptySet();
            if (certificateChecking == null || certificateChecking.getBasicConstraints() != -1) {
                LDAPCertStore.LDAPRequest o2 = new LDAPCertStore.LDAPRequest(name);
                o2.addRequestedAttribute("crossCertificatePair;binary");
                o2.addRequestedAttribute("cACertificate;binary");
                o2.addRequestedAttribute("authorityRevocationList;binary");
                if (this.prefetchCRLs) {
                    o2.addRequestedAttribute("certificateRevocationList;binary");
                }
                try {
                    c = this.getCRLs(o2, "authorityRevocationList;binary", x509CRLSelector);
                    if (c.isEmpty()) {
                        this.prefetchCRLs = true;
                    }
                    else {
                        o.addAll(c);
                    }
                }
                catch (CertStoreException obj2) {
                    if (LDAPCertStore.debug != null) {
                        LDAPCertStore.debug.println("LDAPCertStore.engineGetCRLs non-fatal error retrieving ARLs:" + obj2);
                        obj2.printStackTrace();
                    }
                }
            }
            if (c.isEmpty() || certificateChecking == null) {
                LDAPCertStore.LDAPRequest o3 = new LDAPCertStore.LDAPRequest(name);
                o3.addRequestedAttribute("certificateRevocationList;binary");
                o.addAll(this.getCRLs(o3, "certificateRevocationList;binary", x509CRLSelector));
            }
            else {
                continue;
            }
        }
        return o;
    }
    
    static LDAPCertStoreParameters getParameters(URI uri) {
        String host = uri.getHost();
        if (host == null) {
            return new LDAPCertStore.SunLDAPCertStoreParameters();
        }
        int port = uri.getPort();
        return (port == -1) ? new SunLDAPCertStoreParameters(host) : new LDAPCertStore.SunLDAPCertStoreParameters(host, port);
    }
    
    static {
        debug = Debug.getInstance("certpath");
        STRING0 = new String[0];
        BB0 = new byte[0][];
        EMPTY_ATTRIBUTES = new BasicAttributes();
        String s = (String)AccessController.doPrivileged(new GetPropertyAction("sun.security.certpath.ldap.cache.lifetime"));
        if (s != null) {
            LIFETIME = Integer.parseInt(s);
        }
        else {
            LIFETIME = 30;
        }
        certStoreCache = Cache.newSoftMemoryCache(185);
    }
    
    private class LDAPRequest
    {
        private final String name;
        private Map<String, byte[][]> valueMap;
        private final List<String> requestedAttributes;
        
        LDAPRequest(String s) throws CertStoreException {
            this.name = this.checkName(s);
            this.requestedAttributes = new ArrayList(5);
        }
        
        private String checkName(String str) throws CertStoreException {
            if (str == null) {
                throw new CertStoreException("Name absent");
            }
            try {
                if (new CompositeName(str).size() > 1) {
                    throw new CertStoreException("Invalid name: " + str);
                }
            }
            catch (InvalidNameException cause) {
                throw new CertStoreException("Invalid name: " + str, cause);
            }
            return str;
        }
        
        String getName() {
            return this.name;
        }
        
        void addRequestedAttribute(String s) {
            if (this.valueMap != null) {
                throw new IllegalStateException("Request already sent");
            }
            this.requestedAttributes.add(s);
            return;
        }
        
        byte[][] getValues(String str) throws NamingException {
            byte[][] array = (byte[][])LDAPCertStore.this.valueCache.get(this.name + "|" + str);
            if (array != null) {
                LDAPCertStore.this.cacheHits++;
                return array;
            }
            LDAPCertStore.this.cacheMisses++;
            return (byte[][])this.getValueMap().get(str);
        }
        
        private Map<String, byte[][]> getValueMap() throws NamingException {
            if (this.valueMap != null) {
                return this.valueMap;
            }
            this.valueMap = new HashMap(8);
            String[] array = (String[])this.requestedAttributes.toArray(LDAPCertStore.STRING0);
            Attributes attributes;
            try {
                attributes = LDAPCertStore.this.ctx.getAttributes(this.name, array);
            }
            catch (LdapReferralException ex) {
                while (true) {
                    try {
                        URI o = new URI((String)ex.getReferralInfo());
                        if (!o.getScheme().equalsIgnoreCase("ldap")) {
                            throw new IllegalArgumentException("Not LDAP");
                        }
                        String s = o.getPath();
                        if (s != null && s.charAt(0) == '/') {
                            s = s.substring(1);
                        }
                        this.checkName(s);
                    }
                    catch (Exception ex2) {
                        throw new NamingException("Cannot follow referral to " + ex.getReferralInfo());
                    }
                    LdapContext ldapContext = (LdapContext)ex.getReferralContext();
                    try {
                        attributes = ldapContext.getAttributes(this.name, array);
                    }
                    catch (LdapReferralException ex3) {
                        ex = ex3;
                    }
                    finally {
                        ldapContext.close();
                    }
                }
            }
            catch (NameNotFoundException ex4) {
                attributes = LDAPCertStore.EMPTY_ATTRIBUTES;
            }
            for (String s2 : this.requestedAttributes) {
                byte[][] attributeValues = this.getAttributeValues(attributes.get(s2));
                this.cacheAttribute(s2, attributeValues);
                this.valueMap.put(s2, attributeValues);
            }
            return this.valueMap;
        }
        
        private void cacheAttribute(String str, byte[][] array) {
            LDAPCertStore.this.valueCache.put(this.name + "|" + str, array);
        }
        
        private byte[][] getAttributeValues(Attribute attribute) throws NamingException {
            if (attribute == null) {
                return LDAPCertStore.BB0;
            }
            byte[][] access$600 = new byte[attribute.size()][];
            int n = 0;
            Object next;
            for (NamingEnumeration all = attribute.getAll(); all.hasMore(); access$600[n++] = (byte[])(byte[])next) {
                next = all.next();
                if (LDAPCertStore.debug != null && next instanceof String) {
                    LDAPCertStore.debug.println("LDAPCertStore.getAttrValues() enum.next is a string!: " + next);
                }
            }
            return access$600;
        }
    }
    
    private static class SunLDAPCertStoreParameters extends LDAPCertStoreParameters
    {
        private volatile int hashCode;
        
        SunLDAPCertStoreParameters(String serverName, int port) {
            super(serverName, port);
            this.hashCode = 0;
        }
        
        SunLDAPCertStoreParameters(String serverName) {
            super(serverName);
            this.hashCode = 0;
        }
        
        SunLDAPCertStoreParameters() {
            this.hashCode = 0;
        }
        
        @Override
        public boolean equals(Object o) {
            if (!(o instanceof LDAPCertStoreParameters)) {
                return false;
            }
            LDAPCertStoreParameters ldapCertStoreParameters = (LDAPCertStoreParameters)o;
            return this.getPort() == ldapCertStoreParameters.getPort() && this.getServerName().equalsIgnoreCase(ldapCertStoreParameters.getServerName());
        }
        
        @Override
        public int hashCode() {
            if (this.hashCode == 0) {
                this.hashCode = 37 * (37 * 17 + this.getPort()) + this.getServerName().toLowerCase(Locale.ENGLISH).hashCode();
            }
            return this.hashCode;
        }
    }
    
    static class LDAPCertSelector extends X509CertSelector
    {
        private X500Principal certSubject;
        private X509CertSelector selector;
        private X500Principal subject;
        
        LDAPCertSelector(X509CertSelector x509CertSelector, X500Principal certSubject, String dname) throws IOException {
            this.selector = ((x509CertSelector == null) ? new X509CertSelector() : x509CertSelector);
            this.certSubject = certSubject;
            this.subject = new X500Name(dname).asX500Principal();
        }
        
        @Override
        public X509Certificate getCertificate() {
            return this.selector.getCertificate();
        }
        
        @Override
        public BigInteger getSerialNumber() {
            return this.selector.getSerialNumber();
        }
        
        @Override
        public X500Principal getIssuer() {
            return this.selector.getIssuer();
        }
        
        @Override
        public String getIssuerAsString() {
            return this.selector.getIssuerAsString();
        }
        
        @Override
        public byte[] getIssuerAsBytes() throws IOException {
            return this.selector.getIssuerAsBytes();
        }
        
        @Override
        public X500Principal getSubject() {
            return this.subject;
        }
        
        @Override
        public String getSubjectAsString() {
            return this.subject.getName();
        }
        
        @Override
        public byte[] getSubjectAsBytes() throws IOException {
            return this.subject.getEncoded();
        }
        
        @Override
        public byte[] getSubjectKeyIdentifier() {
            return this.selector.getSubjectKeyIdentifier();
        }
        
        @Override
        public byte[] getAuthorityKeyIdentifier() {
            return this.selector.getAuthorityKeyIdentifier();
        }
        
        @Override
        public Date getCertificateValid() {
            return this.selector.getCertificateValid();
        }
        
        @Override
        public Date getPrivateKeyValid() {
            return this.selector.getPrivateKeyValid();
        }
        
        @Override
        public String getSubjectPublicKeyAlgID() {
            return this.selector.getSubjectPublicKeyAlgID();
        }
        
        @Override
        public PublicKey getSubjectPublicKey() {
            return this.selector.getSubjectPublicKey();
        }
        
        @Override
        public boolean[] getKeyUsage() {
            return this.selector.getKeyUsage();
        }
        
        @Override
        public Set<String> getExtendedKeyUsage() {
            return this.selector.getExtendedKeyUsage();
        }
        
        @Override
        public boolean getMatchAllSubjectAltNames() {
            return this.selector.getMatchAllSubjectAltNames();
        }
        
        @Override
        public Collection<List<?>> getSubjectAlternativeNames() {
            return this.selector.getSubjectAlternativeNames();
        }
        
        @Override
        public byte[] getNameConstraints() {
            return this.selector.getNameConstraints();
        }
        
        @Override
        public int getBasicConstraints() {
            return this.selector.getBasicConstraints();
        }
        
        @Override
        public Set<String> getPolicy() {
            return this.selector.getPolicy();
        }
        
        @Override
        public Collection<List<?>> getPathToNames() {
            return this.selector.getPathToNames();
        }
        
        @Override
        public boolean match(Certificate cert) {
            this.selector.setSubject(this.certSubject);
            int match = this.selector.match(cert);
            this.selector.setSubject(this.subject);
            return match;
        }
    }
    
    static class LDAPCRLSelector extends X509CRLSelector
    {
        private X509CRLSelector selector;
        private Collection<X500Principal> certIssuers;
        private Collection<X500Principal> issuers;
        private HashSet<Object> issuerNames;
        
        LDAPCRLSelector(X509CRLSelector x509CRLSelector, Collection<X500Principal> certIssuers, String s) throws IOException {
            this.selector = ((x509CRLSelector == null) ? new X509CRLSelector() : x509CRLSelector);
            this.certIssuers = certIssuers;
            (this.issuerNames = new HashSet()).add(s);
            (this.issuers = new HashSet()).add(new X500Name(s).asX500Principal());
        }
        
        @Override
        public Collection<X500Principal> getIssuers() {
            return Collections.unmodifiableCollection(this.issuers);
        }
        
        @Override
        public Collection<Object> getIssuerNames() {
            return Collections.unmodifiableCollection(this.issuerNames);
        }
        
        @Override
        public BigInteger getMinCRL() {
            return this.selector.getMinCRL();
        }
        
        @Override
        public BigInteger getMaxCRL() {
            return this.selector.getMaxCRL();
        }
        
        @Override
        public Date getDateAndTime() {
            return this.selector.getDateAndTime();
        }
        
        @Override
        public X509Certificate getCertificateChecking() {
            return this.selector.getCertificateChecking();
        }
        
        @Override
        public boolean match(CRL crl) {
            this.selector.setIssuers(this.certIssuers);
            int match = this.selector.match(crl);
            this.selector.setIssuers(this.issuers);
            return match;
        }
    }
}
